@@ -1,6 +1,7 @@
 /*
  * This File is part of Davix, The IO library for HTTP based protocols
- * Copyright (C) 2013  Adrien Devresse <adrien.devresse@cern.ch>, CERN
+ * Copyright (C) CERN 2013  
+ * Author: Adrien Devresse <adrien.devresse@cern.ch>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -18,7 +19,6 @@
  *
 */
 
-
 #include <utils/davix_types.hpp>
 #include <davix.hpp>
 #include <davix_internal.hpp>
@@ -255,6 +255,9 @@ int authCallbackCert(void* userdata, const SessionInfo & info, X509Credential* c
             
             if( (*err)->getStatus() != StatusCode::CredDecryptionError){
                 // credential specific error
+                std::cerr << "("<< (*err)->getErrScope() <<") Error: "<< (*err)->getErrMsg() << std::endl;
+                DavixError::clearError(err);
+
                 return -1;
             }
 
@@ -268,6 +271,9 @@ int authCallbackCert(void* userdata, const SessionInfo & info, X509Credential* c
                     DavixError::setupError(err, "Davix::Tool::Auth",
                                            StatusCode::CredDecryptionError,
                                            "Impossible to use and decrypt client credential");
+                    
+                    std::cerr << "("<< (*err)->getErrScope() <<") Error: "<< (*err)->getErrMsg() << std::endl;
+                    DavixError::clearError(err);
 
                 }
                 return -1;
@@ -531,7 +537,7 @@ int mkdirP(std::deque<std::string>& dirVec, bool trim){
     // loop backwards and attempt to create dir with largest depth
     // break when an existing dir has been found, or one has been created successfully
     // failed attemps are pushed to queue to retry once we have the location of an existing dir 
-    for(unsigned int i=dirVec.size()-1; i>=0; --i){
+    for(int i=dirVec.size()-1; i>=0; --i){
         ret = mkdir(dirVec[i].c_str(), S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
 
         if((ret == 0) || (errno == EEXIST)){
